/* usplash
 *
 * usplash-theme-jolicloud.c - Jolicloud usplash theme
 *
 * Based on eft theme.
 * Copyright © 2006 Dennis Kaarsemaker <dennis@kaarsemaker.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 */

#include <stdbool.h>
#include <usplash-theme.h>
/* Needed for the custom drawing functions */
#include <usplash_backend.h>
extern struct usplash_pixmap pixmap_picture_install_normal; /* 600x380 */
extern struct usplash_pixmap pixmap_picture_install_compressed; /* 720x380 working area then resized to 600x380 */
extern struct usplash_pixmap pixmap_picture_startup_normal; /* 600x380 */
extern struct usplash_pixmap pixmap_picture_startup_compressed; /* 720x380 working area then resized to 600x380 */
extern struct usplash_pixmap pixmap_picture_shutdown_normal;
extern struct usplash_pixmap pixmap_picture_shutdown_compressed;
extern struct usplash_pixmap pixmap_throbber_pulsating_back_compressed; /* 6x300 working area then resized to 200x300 */
extern struct usplash_pixmap pixmap_throbber_pulsating_fore_compressed;
extern struct usplash_pixmap pixmap_throbber_startup_back_compressed;
extern struct usplash_pixmap pixmap_throbber_startup_fore_compressed;
extern struct usplash_pixmap pixmap_throbber_shutdown_back_compressed;
extern struct usplash_pixmap pixmap_throbber_shutdown_fore_compressed;
extern struct usplash_pixmap pixmap_throbber_blank_back_compressed;

void t_init(struct usplash_theme* theme);
void t_clear_screen(struct usplash_theme* theme);
void t_clear_progressbar(struct usplash_theme* theme);
void t_draw_progressbar(struct usplash_theme* theme, int percentage);
void t_animate_step(struct usplash_theme* theme, int pulsating);

struct usplash_theme usplash_theme;
static int shutting_down = 0;

#define PIXMAP_STARTUP_NORMAL       &pixmap_picture_install_normal
#define PIXMAP_STARTUP_COMPRESSED   &pixmap_picture_install_compressed
#define TOP_EDGE_ADJUST             10
#define PIXMAP_SHUTDOWN_NORMAL      &pixmap_picture_shutdown_normal
#define PIXMAP_SHUTDOWN_COMPRESSED  &pixmap_picture_shutdown_compressed

struct usplash_theme usplash_theme = {
    .version = THEME_VERSION, /* ALWAYS set this to THEME_VERSION, 
                                 it's a compatibility check */
    .next = NULL,
    .ratio = USPLASH_4_3,

    /* Background and font */
    .pixmap = PIXMAP_STARTUP_COMPRESSED,

    /* Theme resolution; if 0, use width/height of pixmap */
    .theme_width = 600,
    .theme_height = 480,

    /* Position of pixmap */
    .pixmap_x = 100,
    .pixmap_y = 5,

    /* Palette indexes */
    .background             = 0,
    .progressbar_background = 0,
    .progressbar_foreground = 0,
    .text_background        = 0,
    .text_foreground        = 6,
    .text_success           = 8,
    .text_failure           = 70,

    /* Progress bar position and size in pixels */
    .progressbar_x      = 200,
    .progressbar_y      = 435,
    .progressbar_width  = 200,
    .progressbar_height = 30,

    /* Text box position and size in pixels */
    .text_x      = 120,
    .text_y      = 465,
    .text_width  = 400,
    .text_height = 15,

    /* Text details */
    .line_height  = 15,
    .line_length  = 32,
    .status_width = 35,

    /* Functions */
    .init = t_init,
    .clear_screen = t_clear_screen,
    .clear_progressbar = t_clear_progressbar,
    .draw_progressbar = t_draw_progressbar,
    .animate_step = t_animate_step,
};

void t_init(struct usplash_theme *theme) {
    int xres, yres;
    usplash_getdimensions(&xres, &yres);
    theme->progressbar_x = (xres - usplash_theme_width(theme))/2 + theme->progressbar_x;
    theme->progressbar_y = (yres - usplash_theme_height(theme))/2 + theme->progressbar_y;    
}

void t_clear_screen(struct usplash_theme *theme) {
    if (shutting_down) {
        if (theme->ratio == USPLASH_16_9)
            theme->pixmap = PIXMAP_SHUTDOWN_NORMAL;
        else
            theme->pixmap = PIXMAP_SHUTDOWN_COMPRESSED;
    }
    else {
        if (theme->ratio == USPLASH_16_9)
            theme->pixmap = PIXMAP_STARTUP_NORMAL;
        else
            theme->pixmap = PIXMAP_STARTUP_COMPRESSED;
    }
    int xres, yres, left_edge, top_edge;
    usplash_getdimensions(&xres, &yres);
    left_edge = (xres - theme->pixmap->width) / 2;
    top_edge = ((yres - theme->pixmap->height) / 2) - TOP_EDGE_ADJUST;
    usplash_clear(0, 0, xres, yres, theme->background);
    usplash_put(left_edge, top_edge, theme->pixmap);
}

void t_clear_progressbar(struct usplash_theme *theme) {
#ifndef INSTALL
    usplash_put(theme->progressbar_x, theme->progressbar_y, &pixmap_throbber_blank_back_compressed);
#endif
}

void t_draw_progressbar(struct usplash_theme *theme, int percentage) {
#ifndef INSTALL
    int w = (pixmap_throbber_startup_back_compressed.width * percentage / 100);

    if (percentage == 0)
        usplash_put( theme->progressbar_x, theme->progressbar_y,
                &pixmap_throbber_blank_back_compressed );

    if (percentage < 0) {/* Unloading */
        if (!shutting_down) {
            shutting_down = 1;
            clear_screen();
        }
        w *= -1;

        /* Draw background to left of foreground */
        usplash_put_part(theme->progressbar_x, theme->progressbar_y, w, pixmap_throbber_shutdown_back_compressed.height, 
                         &pixmap_throbber_shutdown_back_compressed, 0, 0);
        /* Draw foreground to right of background */
        usplash_put_part(theme->progressbar_x + w, theme->progressbar_y, pixmap_throbber_shutdown_back_compressed.width - w,
                         pixmap_throbber_shutdown_back_compressed.height, &pixmap_throbber_shutdown_fore_compressed, w, 0);
    }
    else {/* Loading */
        if (shutting_down) {
            shutting_down = 0;
            clear_screen();
        }

        /* Draw foreground to left of background */
        usplash_put_part(theme->progressbar_x, theme->progressbar_y, w, pixmap_throbber_startup_back_compressed.height, 
                         &pixmap_throbber_startup_fore_compressed, 0, 0);
        /* Draw background ot right of foreground */
        usplash_put_part(theme->progressbar_x + w, theme->progressbar_y, pixmap_throbber_startup_back_compressed.width - w,
                         pixmap_throbber_startup_back_compressed.height, &pixmap_throbber_startup_back_compressed, w, 0);
    }
#endif
}

void t_animate_step(struct usplash_theme* theme, int pulsating) {
#ifndef INSTALL
    static int pulsate_step = 0;
    static int pulse_width = 16;
    static int step_width = 2;
    static int num_steps = 0;
    int x1;
    int x2;
    num_steps = (pixmap_throbber_pulsating_fore_compressed.width - pulse_width)/2;

    if (pulsating) {
        if (pulsate_step < num_steps/2+1) {
            x1 = 2 * step_width * pulsate_step;
        }
        else {
            x1 = pixmap_throbber_pulsating_fore_compressed.width - pulse_width - 2 * step_width * (pulsate_step - num_steps/2+1);
        }
        x2 = x1 + pulse_width;

        /* Draw progress bar background on left side of foreground 'pulse' */
        usplash_put_part(theme->progressbar_x, theme->progressbar_y, x1,
                         pixmap_throbber_pulsating_back_compressed.height, &pixmap_throbber_pulsating_back_compressed, 0, 0);
        /* Draw progress bar foreground 'pulse' */
        usplash_put_part(theme->progressbar_x + x1, theme->progressbar_y, pulse_width,
                         pixmap_throbber_pulsating_back_compressed.height, &pixmap_throbber_pulsating_fore_compressed, x1, 0);
        /* Draw progress bar background on right side of foreground 'pulse' */
        usplash_put_part(theme->progressbar_x + x2, theme->progressbar_y, pixmap_throbber_pulsating_back_compressed.width - x2,
                         pixmap_throbber_pulsating_back_compressed.height, &pixmap_throbber_pulsating_back_compressed, x2, 0);

        pulsate_step = (pulsate_step + 1) % num_steps;
    }
#endif
}
